@@ -1,8 +1,7 @@
 from django.contrib.contenttypes.generic import BaseGenericInlineFormSet, GenericInlineModelAdmin
-from django.contrib.contenttypes.models import ContentType
-from fluent_contents import extensions
+from fluent_contents import extensions, appsettings
 from fluent_contents.forms import ContentItemForm
-from fluent_contents.models.db import Placeholder
+from fluent_contents.models import Placeholder, get_parent_language_code
 
 
 class BaseContentItemFormSet(BaseGenericInlineFormSet):
@@ -10,6 +9,14 @@ class BaseContentItemFormSet(BaseGenericInlineFormSet):
     Correctly save placeholder fields.
     """
     def __init__(self, *args, **kwargs):
+        instance = kwargs['instance']
+        if instance:
+            self.current_language = get_parent_language_code(instance)
+            if self.current_language and 'queryset' in kwargs:
+                kwargs['queryset'] = kwargs['queryset'].filter(language_code=self.current_language)
+        else:
+            self.current_language = appsettings.FLUENT_CONTENTS_DEFAULT_LANGUAGE_CODE
+
         super(BaseContentItemFormSet, self).__init__(*args, **kwargs)
         self._deleted_placeholders = ()  # internal property, set by PlaceholderEditorAdmin
 
@@ -24,7 +31,16 @@ class BaseContentItemFormSet(BaseGenericInlineFormSet):
         # This allows it to insert the parent relation (ct_field / ct_fk_field) since they don't exist in the form.
         # It then calls form.cleaned_data to get all values, and use Field.save_form_data() to store them in the model instance.
         self._set_placeholder_id(form)
-        return super(BaseContentItemFormSet, self).save_new(form, commit=commit)
+
+        # As save_new() completely circumvents form.save(), have to insert the language code here.
+        instance = super(BaseContentItemFormSet, self).save_new(form, commit=False)
+        instance.language_code = self.current_language
+
+        if commit:
+            instance.save()
+            form.save_m2m()
+
+        return instance
 
 
     def save_existing(self, form, instance, commit=True):
@@ -40,12 +56,7 @@ class BaseContentItemFormSet(BaseGenericInlineFormSet):
             form_placeholder = form.cleaned_data['placeholder']   # could already be updated, or still point to previous placeholder.
 
             if not form_placeholder or form_placeholder.slot != form_slot:
-                desired_placeholder = Placeholder.objects.get(
-                    slot=form_slot,
-                    parent_type=ContentType.objects.get_for_model(self.instance),
-                    parent_id=self.instance.id
-                )
-
+                desired_placeholder = Placeholder.objects.parent(self.instance).get(slot=form_slot)
                 form.cleaned_data['placeholder'] = desired_placeholder
                 form.instance.placeholder = desired_placeholder
 
