@@ -2,12 +2,15 @@
 Internal module for the plugin system,
 the API is exposed via __init__.py
 """
+import six
+
+from future.builtins import str
 from threading import Lock
-from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
-from django.utils.importlib import import_module
+from fluent_contents import appsettings
 from fluent_contents.forms import ContentItemForm
 from fluent_contents.models import ContentItem
+from fluent_utils.load import import_apps_submodule
 from .pluginbase import ContentPlugin
 
 
@@ -22,20 +25,6 @@ __all__ = ('PluginContext', 'ContentPlugin', 'plugin_pool')
 # Naturally, you want STATIC_URL to be available in plugins.
 
 
-def _import_apps_submodule(submodule):
-    """
-    Look for a submodule is a series of packages, e.g. ".content_plugins" in all INSTALLED_APPS.
-    """
-    for app in settings.INSTALLED_APPS:
-        try:
-            import_module('.' + submodule, app)
-        except ImportError, e:
-            if submodule not in str(e):
-                raise   # import error is a level deeper.
-            else:
-                pass
-
-
 class PluginAlreadyRegistered(Exception):
     """
     Raised when attemping to register a plugin twice.
@@ -43,6 +32,13 @@ class PluginAlreadyRegistered(Exception):
     pass
 
 
+class ModelAlreadyRegistered(Exception):
+    """
+    The model is already registered with another plugin.
+    """
+    pass
+
+
 class PluginNotFound(Exception):
     """
     Raised when the plugin could not be found in the rendering process.
@@ -78,11 +74,23 @@ class PluginPool(object):
         assert issubclass(plugin.model, ContentItem), "The plugin model must inherit from `ContentItem`"
         assert issubclass(plugin.form, ContentItemForm), "The plugin form must inherit from `ContentItemForm`"
 
-        name = plugin.__name__
+        name = plugin.__name__  # using class here, no instance created yet.
         if name in self.plugins:
             raise PluginAlreadyRegistered("{0}: a plugin with this name is already registered".format(name))
         name = name.lower()
 
+        # Avoid registering 2 plugins to the exact same model. If you want to reuse code, use proxy models.
+        if plugin.model in self._name_for_model:
+            # Having 2 plugins for one model breaks ContentItem.plugin and the frontend code
+            # that depends on using inline-model names instead of plugins. Good luck fixing that.
+            # Better leave the model==plugin dependency for now.
+            existing_plugin = self.plugins[self._name_for_model[plugin.model]]
+            raise ModelAlreadyRegistered("Can't register the model {0} to {2}, it's already registered to {1}!".format(
+                plugin.model.__name__,
+                existing_plugin.name,
+                plugin.__name__
+            ))
+
         # Make a single static instance, similar to ModelAdmin.
         plugin_instance = plugin()
         self.plugins[name] = plugin_instance
@@ -100,7 +108,23 @@ class PluginPool(object):
         Return the list of all plugin instances which are loaded.
         """
         self._import_plugins()
-        return self.plugins.values()
+        return list(self.plugins.values())
+
+
+    def get_allowed_plugins(self, placeholder_slot):
+        """
+        Return the plugins which are supported in the given placeholder name.
+        """
+        # See if there is a limit imposed.
+        slot_config = appsettings.FLUENT_CONTENTS_PLACEHOLDER_CONFIG.get(placeholder_slot) or {}
+        plugins = slot_config.get('plugins')
+        if not plugins:
+            return self.get_plugins()
+        else:
+            try:
+                return self.get_plugins_by_name(*plugins)
+            except PluginNotFound as e:
+                raise PluginNotFound(str(e) + " Update the plugin list of the FLUENT_CONTENTS_PLACEHOLDER_CONFIG['{0}'] setting.".format(placeholder_slot))
 
 
     def get_plugins_by_name(self, *names):
@@ -110,7 +134,7 @@ class PluginPool(object):
         self._import_plugins()
         plugin_instances = []
         for name in names:
-            if isinstance(name, basestring):
+            if isinstance(name, six.string_types):
                 try:
                     plugin_instances.append(self.plugins[name.lower()])
                 except KeyError:
@@ -180,7 +204,7 @@ class PluginPool(object):
             return  # previous threaded released + completed
 
         try:
-            _import_apps_submodule("content_plugins")
+            import_apps_submodule("content_plugins")
             self.detected = True
         finally:
             self.scanLock.release()
@@ -191,7 +215,7 @@ class PluginPool(object):
         if self._name_for_ctype_id is None:
             plugin_ctypes = {}  # separate dict to build, thread safe
             self._import_plugins()
-            for name, plugin in self.plugins.iteritems():
+            for name, plugin in self.plugins.items():
                 plugin_ctypes[plugin.type_id] = name
 
             self._name_for_ctype_id = plugin_ctypes
