@@ -1,9 +1,10 @@
 import django
-from django.contrib.contenttypes.generic import GenericRelation, GenericRel
 from django.contrib.contenttypes.models import ContentType
+from django.db import DEFAULT_DB_ALIAS
 from django.db.models.query_utils import Q
 from django.utils.functional import lazy
 from django.utils.text import capfirst
+from fluent_utils.django_compat.moves.contenttypes import GenericRelation, GenericRel
 from fluent_contents import appsettings
 from fluent_contents.forms.fields import PlaceholderFormField
 from fluent_contents.models import Placeholder, ContentItem
@@ -53,11 +54,18 @@ class ContentItemRelation(GenericRelation):
 
         class Page(models.Model):
             contentitem_set = ContentItemRelation()
+
+    Adding this relation also causes the admin delete page to list the
+    :class:`~fluent_contents.models.ContentItem` objects which will be deleted.
     """
     def __init__(self, **kwargs):
         super(ContentItemRelation, self).__init__(to=ContentItem,
             object_id_field='parent_id', content_type_field='parent_type', **kwargs)
 
+    def bulk_related_objects(self, objs, using=DEFAULT_DB_ALIAS):
+        # Fix delete screen. Workaround for https://github.com/chrisglass/django_polymorphic/issues/34
+        return super(ContentItemRelation, self).bulk_related_objects(objs).non_polymorphic()
+
 
 class PlaceholderRel(GenericRel):
     """
@@ -102,7 +110,7 @@ class PlaceholderFieldDescriptor(object):
         if instance is None:
             return self
         try:
-            return Placeholder.objects.get_by_slot(instance, self.slot)
+            placeholder = Placeholder.objects.get_by_slot(instance, self.slot)
         except Placeholder.DoesNotExist:
             raise Placeholder.DoesNotExist("Placeholder does not exist for parent {0} (type_id: {1}, parent_id: {2}), slot: '{3}'".format(
                 repr(instance),
@@ -110,6 +118,9 @@ class PlaceholderFieldDescriptor(object):
                 instance.pk,
                 self.slot
             ))
+        else:
+            placeholder.parent = instance  # fill the reverse cache
+            return placeholder
 
 
     def __set__(self, instance, value):
@@ -239,16 +250,17 @@ class PlaceholderField(PlaceholderRelation):
 
 
 
-try:
-    from south.modelsinspector import add_ignored_fields
-except ImportError:
-    pass
-else:
-    # South 0.7.x ignores GenericRelation fields but doesn't ignore subclasses.
-    # Taking the same fix as applied in http://south.aeracode.org/ticket/414
-    _name_re = "^" + __name__.replace(".", "\.")
-    add_ignored_fields((
-        _name_re + "\.PlaceholderField",
-        _name_re + "\.PlaceholderRelation",
-        _name_re + "\.ContentItemRelation",
-    ))
+if django.VERSION < (1,7):
+    try:
+        from south.modelsinspector import add_ignored_fields
+    except ImportError:
+        pass
+    else:
+        # South 0.7.x ignores GenericRelation fields but doesn't ignore subclasses.
+        # Taking the same fix as applied in http://south.aeracode.org/ticket/414
+        _name_re = "^" + __name__.replace(".", "\.")
+        add_ignored_fields((
+            _name_re + "\.PlaceholderField",
+            _name_re + "\.PlaceholderRelation",
+            _name_re + "\.ContentItemRelation",
+        ))
