@@ -1,12 +1,21 @@
+from future.builtins import str
+from future.utils import python_2_unicode_compatible
+from django.core.exceptions import ValidationError
 from django.db import models
 from django.utils.translation import ugettext_lazy as _
+from micawber import ProviderException
 from fluent_contents.models.db import ContentItem
 from fluent_contents.plugins.oembeditem.fields import OEmbedUrlField
 from fluent_contents.plugins.oembeditem import backend
 
-class OEmbedItem(ContentItem):
+
+@python_2_unicode_compatible
+class AbstractOEmbedItem(ContentItem):
     """
-    Embedded media via OEmbed
+    The base class for an OEmbedItem,
+    This allows to create custom models easily.
+
+    .. versionadded:: 1.0
     """
     TYPE_PHOTO = 'photo'
     TYPE_VIDEO = 'video'
@@ -39,35 +48,73 @@ class OEmbedItem(ContentItem):
 
 
     class Meta:
+        abstract = True
         verbose_name = _("Online media")
         verbose_name_plural = _("Online media")
 
 
-    def __unicode__(self):
+    def __str__(self):
         return self.title or self.embed_url
 
 
     def __init__(self, *args, **kwargs):
-        super(OEmbedItem, self).__init__(*args, **kwargs)
+        super(AbstractOEmbedItem, self).__init__(*args, **kwargs)
         self._old_embed_url = self.embed_url
         self._old_embed_max_width = self.embed_max_width
         self._old_embed_max_height = self.embed_max_height
 
 
     def save(self, *args, **kwargs):
-        self.update_oembed_data()
-        super(OEmbedItem, self).save(*args, **kwargs)
+        self.update_oembed_data()  # if clean() did not run, still update the oembed
+        super(AbstractOEmbedItem, self).save(*args, **kwargs)
 
 
-    def update_oembed_data(self):
-        if not self.type \
-        or self._old_embed_url != self.embed_url \
-        or self._old_embed_max_width != self.embed_max_width \
-        or self._old_embed_max_height != self.embed_max_height:
+    def clean(self):
+        # Avoid getting server errors when the URL is not valid.
+        try:
+            self.update_oembed_data()
+        except ProviderException as e:
+            raise ValidationError(str(e))
+
+
+    def update_oembed_data(self, force=False, **backend_params):
+        """
+        Update the OEmbeddata if needed.
+
+        .. versionadded:: 1.0 Added force and backend_params parameters.
+        """
+        if force or self._input_changed():
             # Fetch new embed code
-            response = backend.get_oembed_data(self.embed_url, self.embed_max_width, self.embed_max_height)
+            params = self.get_oembed_params(self.embed_url)
+            params.update(backend_params)
+            response = backend.get_oembed_data(self.embed_url, **params)
+
+            # Save it
             self.store_response(response)
+
+            # Track field changes
             self._old_embed_url = self.embed_url
+            self._old_embed_max_width = self.embed_max_width
+            self._old_embed_max_height = self.embed_max_height
+
+
+    def get_oembed_params(self, embed_url):
+        """
+        .. versionadded:: 1.0
+
+           Allow to define the parameters that are passed to the backend to fetch the current URL.
+        """
+        return {
+            'max_width': self.embed_max_width,
+            'max_height': self.embed_max_height,
+        }
+
+
+    def _input_changed(self):
+        return not self.type \
+            or self._old_embed_url != self.embed_url \
+            or self._old_embed_max_width != self.embed_max_width \
+            or self._old_embed_max_height != self.embed_max_height
 
 
     def store_response(self, response):
@@ -80,5 +127,16 @@ class OEmbedItem(ContentItem):
         )
 
         for key in KEYS:
-            if response.has_key(key):
+            if key in response:
                 setattr(self, key, response[key])
+
+
+
+class OEmbedItem(AbstractOEmbedItem):
+    """
+    Embedded media via OEmbed
+    """
+
+    class Meta:
+        verbose_name = _("Online media")
+        verbose_name_plural = _("Online media")
