@@ -1,3 +1,4 @@
+
 from django.contrib.admin.widgets import AdminTextareaWidget
 from django.utils.translation import ugettext_lazy as _
 from fluent_contents.extensions import ContentPlugin, plugin_pool
@@ -21,3 +22,10 @@ class PicturePlugin(ContentPlugin):
     radio_fields = {
         'align': ContentPlugin.HORIZONTAL
     }
+
+    class Media:
+        css = {
+            'screen': (
+                'fluent_contents/plugins/picture/picture_admin.css',
+            )
+        }
