@@ -51,6 +51,7 @@ var cp_plugins = {};
       // jQuery 1.7+
       $("#content-main")
         .on('click', ".cp-plugin-add-button", cp_plugins.onAddButtonClick )
+        .on('click', ".cp-copy-language-controls .cp-copy-button", cp_plugins.onCopyLanguageButtonClick )
         .on('click', ".cp-item-controls .cp-item-up", cp_plugins.onItemUpClick )
         .on('click', ".cp-item-controls .cp-item-down", cp_plugins.onItemDownClick )
         .on('click', ".cp-item-controls .cp-item-move", cp_plugins.onItemMoveClick )
@@ -58,6 +59,7 @@ var cp_plugins = {};
     }
     else {
       $(".cp-plugin-add-button").live( 'click', cp_plugins.onAddButtonClick );
+      $(".cp-copy-language-controls .cp-copy-button").live( 'click', cp_plugins.onCopyLanguageButtonClick );
       $(".cp-item-controls .cp-item-up").live( 'click', cp_plugins.onItemUpClick );
       $(".cp-item-controls .cp-item-down").live( 'click', cp_plugins.onItemDownClick );
       $(".cp-item-controls .cp-item-move").live( 'click', cp_plugins.onItemMoveClick );
@@ -96,7 +98,11 @@ var cp_plugins = {};
     // Count number of seen tabs per role.
     var roles_seen = {};
     var placeholders = cp_data.get_placeholders();
-    for(var i in placeholders)
+    if(placeholders == null) {
+      console.error("Placeholders are not defined. Is the proper PlaceholderFieldAdmin/PlaceholderEditorAdmin class used?")
+      return;
+    }
+    for(var i = 0; i < placeholders.length; i++)
       roles_seen[placeholders[i].role] = 0;
 
     // Move all items to the tabs.
@@ -105,6 +111,9 @@ var cp_plugins = {};
     // Depending on the current layout, it placeholder pane can be found, or needs to be migrated.
     for(var placeholder_slot in cp_data.dom_placeholders)
     {
+      if(! cp_data.dom_placeholders.hasOwnProperty(placeholder_slot))
+        continue;
+
       var dom_placeholder = cp_data.dom_placeholders[placeholder_slot];
       var last_role_occurance =  ++roles_seen[dom_placeholder.role];
       if( dom_placeholder.items.length == 0)
@@ -129,6 +138,7 @@ var cp_plugins = {};
    */
   cp_plugins.get_new_placeholder_pane = function(dom_placeholder, last_known_nr)
   {
+    var pane;
     var isExpiredTab = window.cp_tabs ? cp_tabs.is_expired_tab : function(node) { return false; };
 
     // Option 1. Find identical placeholder by slot name.
@@ -136,7 +146,7 @@ var cp_plugins = {};
     if( placeholder )
     {
       // Option 1. Find by slot name,
-      var pane = cp_data.get_placeholder_pane(placeholder);
+      pane = cp_data.get_placeholder_pane(placeholder);
       if( pane && ! isExpiredTab(pane.root) )
         return pane;
     }
@@ -153,12 +163,29 @@ var cp_plugins = {};
       }
     }
 
-    // Option 3. Open a "lost+found" tab.
+    // Option 3: If there is only one placeholder, that can be used.
+    // This is typically the case for the PlaceholderField() object at the page.
+    var single_placeholder = cp_data.get_single_placeholder();
+    if( single_placeholder )
+    {
+      pane = cp_data.get_placeholder_pane(single_placeholder);
+      if( pane && ! isExpiredTab(pane.root) )
+      {
+        console.log("Using single placeholder '" + single_placeholder.slot + "' as fallback for item from placeholder '" + dom_placeholder.slot + "'.");
+        return pane;
+      }
+    }
+
+    // Option 4. Open a "lost+found" tab.
     // NOTE: not really a clean solution, needs a better (public) API for this (cp_tabs is optional).
     if( window.cp_tabs )
     {
-      console.log("Using orphaned tab as fallback for item from placeholder '" + dom_placeholder.slot + "'.");
-      return cp_tabs.get_fallback_pane();
+      pane = cp_tabs.get_fallback_pane();
+      if( pane )
+      {
+        console.log("Using orphaned tab as fallback for item from placeholder '" + dom_placeholder.slot + "'.");
+        return pane;
+      }
     }
 
     throw new Error("No placeholder pane for placeholder: " + dom_placeholder.slot + " (role: " + dom_placeholder.role + ")");
@@ -184,7 +211,7 @@ var cp_plugins = {};
 
     // Move all items to that tab.
     // Restore item values upon restoring fields.
-    for(var i in dom_placeholder.items)
+    for(var i = 0; i < dom_placeholder.items.length; i++)
     {
       var $fs_item = dom_placeholder.items[i];
       dom_placeholder.items[i] = cp_plugins._move_item_to( $fs_item, function _move_to_pane($fs_item)
@@ -213,7 +240,7 @@ var cp_plugins = {};
 
     // Don't restore the special fields,
     // The add_action could move the formset item, and this update it.
-    var ignoreFields = ['placeholder', 'placeholder_slot', 'sort_order'];
+    var ignoreFields = ['placeholder', 'placeholder_slot', 'sort_order', 'DELETE'];
     var ignoreTest = function(name) { return $.inArray(name.substring(name.lastIndexOf('-')+1), ignoreFields) != -1; };
 
     // Remove the item.
@@ -240,13 +267,13 @@ var cp_plugins = {};
     for(var i = 0; i < $inputs.length; i++)
     {
       var $input = $inputs.eq(i)
-        , name = $input.attr("name");
-
-      if( $input[0].type == 'radio' && !$input[0].checked )
+        , id = $input.attr("id") || $input.attr("name")  // multiple input checkbox has one name, but different IDs
+        , input_type = $input[0].type;
+      if((input_type == 'radio' || input_type == 'checkbox') && !$input[0].checked)
         continue;
 
       if( !ignoreTestFunc || !ignoreTestFunc(name) )
-        values[name] = $input.val();
+        values[id] = $input.val();
     }
 
     return values;
@@ -259,28 +286,35 @@ var cp_plugins = {};
     for(var i = 0; i < $inputs.length; i++)
     {
       var $input = $inputs.eq(i)
-        , name = $input.attr("name");
+        , id = $input.attr("id") || $input.attr("name");
 
-      if( values.hasOwnProperty(name)
-       && (!ignoreTestFunc || !ignoreTestFunc(name)) )
+      if( values.hasOwnProperty(id)
+       && (!ignoreTestFunc || !ignoreTestFunc(id)) )
       {
-        var value = values[name];
-        if( $input[0].type == 'radio' )
-        {
-          $input[0].checked = ($input[0].value == value);
-        }
-        else
-        {
-          if(value == null)
-            $input.removeAttr('value');
-          else
-            $input.val(value);
-        }
+        var value = values[id];
+        cp_plugins._set_input_value($input, value);
       }
     }
   }
 
 
+  cp_plugins._set_input_value = function($input, value)
+  {
+    var input_type = $input[0].type;
+    if( input_type == 'radio' || input_type == 'checkbox' )
+    {
+      $input[0].checked = ($input[0].value == value);
+    }
+    else
+    {
+      if(value == null)
+        $input.removeAttr('value');
+      else
+        $input.val(value);
+    }
+  }
+
+
   // -------- Add plugin feature ------
 
   /**
@@ -298,8 +332,10 @@ var cp_plugins = {};
   /**
    * Add an item to a tab.
    */
-  cp_plugins.add_formset_item = function( placeholder_slot, model_name )
+  cp_plugins.add_formset_item = function( placeholder_slot, model_name, options )
   {
+    options = options || {};
+
     // The Django admin/media/js/inlines.js API is not public, or easy to use.
     // Recoded the inline model dynamics.
 
@@ -316,7 +352,9 @@ var cp_plugins = {};
     // Clone the item.
     var new_index = total.value;
     var item_id   = inline_meta.prefix + "-" + new_index;
-    var newhtml   = inline_meta.item_template.get_outerHtml().replace(/__prefix__/g, new_index);
+    var newhtml   = options.get_html
+                  ? options.get_html(inline_meta, new_index)   // hook
+                  : inline_meta.item_template.get_outerHtml().replace(/__prefix__/g, new_index);
     var $newitem  = $(newhtml).removeClass("empty-form").attr("id", item_id);
 
     // Add it
@@ -335,22 +373,28 @@ var cp_plugins = {};
     cp_plugins._set_pageitem_data($fs_item, placeholder, new_index);
     cp_plugins.enable_pageitem($fs_item);
     cp_plugins.update_sort_order(pane);  // Not required, but keep the form state consistent all the time.
+    if(options.on_post_add) options.on_post_add($fs_item);
   }
 
 
   cp_plugins._set_pageitem_data = function($fs_item, placeholder, new_sort_index)
   {
-    // Currently redetermining group_prefix, avoid letting fs_item to go out of sync with different call paths.
-    var current_item = cp_data.get_inline_formset_item_info($fs_item);
-    var group_prefix = current_item.auto_id.replace(/%s/, current_item.prefix);
-    var field_prefix = group_prefix + "-" + current_item.index;
-
+    var field_prefix = cp_plugins._get_field_prefix($fs_item);
     $("#" + field_prefix + "-placeholder").val(placeholder.id);
     $("#" + field_prefix + "-placeholder_slot").val(placeholder.slot);
     $("#" + field_prefix + "-sort_order").val(new_sort_index);
   }
 
 
+  cp_plugins._get_field_prefix = function($fs_item)
+  {
+    // Currently redetermining group_prefix, avoid letting fs_item to go out of sync with different call paths.
+    var current_item = cp_data.get_inline_formset_item_info($fs_item);
+    var group_prefix = current_item.auto_id.replace(/%s/, current_item.prefix);
+    return group_prefix + "-" + current_item.index;
+  }
+
+
   // -------- Moving and sorting plugins ------
 
 
@@ -364,6 +408,7 @@ var cp_plugins = {};
   cp_plugins.onItemMoveClick = function(event)
   {
     event.preventDefault();
+    event.stopPropagation();
     cp_plugins._show_move_popup(event.target);
   }
 
@@ -399,7 +444,8 @@ var cp_plugins = {};
     var $fs_item = current_item.fs_item;
 
     var old_pane = cp_data.get_placeholder_pane_for_item($fs_item);
-    var old_placeholder = cp_data.get_placeholder_by_slot(dominfo.placeholder_slot);  // slot is always filled in.
+    var old_slot = old_pane.is_orphaned ? '__orphaned__' : dominfo.placeholder_slot;
+    var old_placeholder = cp_data.get_placeholder_by_slot(old_slot);
     var new_placeholder = cp_data.get_placeholder_by_slot(slot);
     var dom_placeholder = cp_data.get_or_create_dom_placeholder(new_placeholder);
     var new_pane = cp_data.get_placeholder_pane(new_placeholder);
@@ -446,9 +492,10 @@ var cp_plugins = {};
     var $child_node = $(child_node);
     var pos = $child_node.offset();
     var $popup = $("#cp-move-popup");
+    var oldJquery = !$.fn.on;  // assume that 1.7 has the bug fixed.
     $popup.offset({
-      left: parseInt(pos.left) - $popup.width() + 18 + $window.scrollLeft(),
-      top: parseInt(pos.top) + $child_node.height() + 2 + $window.scrollTop()
+      left: parseInt(pos.left) - $popup.width() + 18 + (oldJquery ? $window.scrollLeft() : 0),
+      top: parseInt(pos.top) + $child_node.height() + 2 + (oldJquery ? $window.scrollTop() : 0)
     });
 
     // Configure clicks
@@ -553,7 +600,7 @@ var cp_plugins = {};
   cp_plugins._sort_items = function($items)
   {
     // The sort_order field is likely top-level, but the fieldset html can place it anywhere.
-    for( var i in $items)
+    for( var i = 0; i < $items.length; i++ )
     {
       var $fs_item = $items[i];
       $fs_item._sort_order = parseInt($fs_item.find("input[id$=-sort_order]:first").val());
@@ -564,6 +611,71 @@ var cp_plugins = {};
 
 
 
+  // -------- Copy languages ------
+
+  cp_plugins.onCopyLanguageButtonClick = function(event)
+  {
+    var $button = $(event.target);
+    var language_code = $button.siblings('select').val();
+    var placeholder_slot = $button.attr('data-placeholder-slot');
+    var url = $('.inline-placeholder-group').attr('data-get-placeholder-data-url');
+    $.ajax({
+      url: url,
+      dataType: 'json',
+      data: 'language=' + language_code,
+      success: function(data, textStatus, xhr)
+      {
+        // Ask to update the tabs!
+        if(data.success)
+          cp_plugins.load_formset_data(data, placeholder_slot);
+        else
+          alert("Internal CMS error: failed to fetch site data!");
+      },
+      error: function(xhr, textStatus, ex)
+      {
+        alert("Internal CMS error: failed to fetch site data!");    // can't yet rely on $.ajaxError
+      }
+    });
+  }
+
+
+  cp_plugins.load_formset_data = function(data, match_placeholder_slot)
+  {
+    for (var i = 0; i < data.formset_forms.length; i++) {
+      // Each item is stored as basic formdata
+      // and generated HTML.
+      var itemdata = data.formset_forms[i];
+      if(match_placeholder_slot && itemdata.placeholder_slot != match_placeholder_slot)
+        continue;
+
+      // Replace the server-side generated prefix,
+      // as this clearly won't be the same as what we'll generate client-side.
+      var re_old_prefix = new RegExp(itemdata.prefix + '-', 'g');
+
+      cp_plugins.add_formset_item(itemdata.placeholder_slot, itemdata.model, {
+        'get_html': function(inline_meta, new_index) {
+          // Use the server-side provided HTML, which has fields filled in
+          // with all template-styling handled. It's a literal copy of the edit page.
+          var new_prefix = inline_meta.prefix + "-" + new_index + "-";
+          var new_formfields = itemdata.html.replace(re_old_prefix, new_prefix);
+
+          // Take the original template, replace the contents of the 'cp-formset-item-fields' block.
+          var orig_html = inline_meta.item_template.get_outerHtml().replace(/__prefix__/g, new_index);
+          var $orig_html = $(orig_html);
+          $orig_html.find('.cp-formset-item-fields').empty().html(new_formfields);
+          return $orig_html;
+        },
+        'on_post_add': function($fs_item) {
+          // Trigger a change() event for radio fields.
+          // This fixes the django-any-urlfield display.
+          $fs_item.find('input[type=radio]:checked').change();
+        }
+      });
+    }
+  }
+
+
+
   // -------- Delete plugin ------
 
 
