@@ -1,3 +1,4 @@
+from future.utils import python_2_unicode_compatible
 from django.db import models
 from fluent_contents.models import ContentItem, PlaceholderField, PlaceholderRelation, ContentItemRelation
 
@@ -9,10 +10,12 @@ class TestPage(models.Model):
     contents = models.TextField("Contents")
 
     class Meta:
+        app_label = 'testapp'
         verbose_name = "Test page"
         verbose_name_plural = "Test pages"
 
 
+@python_2_unicode_compatible
 class PlaceholderFieldTestPage(models.Model):
     """
     A model with PlaceholderField, for testing,
@@ -24,13 +27,15 @@ class PlaceholderFieldTestPage(models.Model):
     contentitem_set = ContentItemRelation()
 
     class Meta:
+        app_label = 'testapp'
         verbose_name = "Test page"
         verbose_name_plural = "Test pages"
 
-    def __unicode__(self):
+    def __str__(self):
         return self.title
 
 
+@python_2_unicode_compatible
 class RawHtmlTestItem(ContentItem):
     """
     The most basic "raw HTML" content item, for testing.
@@ -38,8 +43,9 @@ class RawHtmlTestItem(ContentItem):
     html = models.TextField("HTML code")
 
     class Meta:
+        app_label = 'testapp'
         verbose_name = 'Test HTML code'
         verbose_name_plural = 'Test HTML codes'
 
-    def __unicode__(self):
+    def __str__(self):
         return self.html
